#!/usr/bin/env python

import sys
import os
import time
from lunet.lubase import *
from lunet.gtkapp import *
from lunet.dpipes import PortFs


##################################################
## Globalne definicie                           ##
##################################################

APPNAME="Duplicator"
APPVER="02.04.1903"
APPTITLE="LUnet Studio Duplicator 2.4"


cfg=ConfConstructor('/etc/duplicator')
conf=cfg.load('defaults.conf')

if conf['cfgHiddenDevs']!='':
	conf.update({'arrHiddenDevs':conf['cfgHiddenDevs'].split(',')})
else:
	conf.update({'arrHiddenDevs':[]})
conf.update({'arrAppLinks':conf['cmdAppLinks'].split('|')})

appdir=conf['pathApp']

cargs=sys.argv
c=LU()
pfs=PortFs('duplicator', dict({'size':'1M'}))

dynglade=DynamicGlade(conf['cfgLang'], conf['pathApp']+'/inc', conf['pathTmp'], conf['pathApp']+'/share/lng')
app=GtkApplication(dynglade.create('main'))

sID=0
threads=[9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
actions=dict({})



##################################################
# Signaly

def sigMenuActions(widget, *args):
	action=widget.get_property('name')
	if action=='actQuit':
		app.quit()
	elif action=='actAbout':
		cw.show('about', {'title':dynglade.terms['about']+' '+APPNAME,\
				  'image':'/usr/share/duplicator/duplicator.png',\
				  'name':'LUnet Studio '+APPNAME,\
				  'version':APPVER,\
				  'description':dynglade.terms['aboutdesc'].replace('<br>', "\n"),\
				  'copyright':'(c) 2015-2019 LUnet Studio, All rights reserved!',\
				  'links':[[APPNAME+' Homepage', 'https://products.lunet.studio/en/duplicator.html'],\
				['Help and Support', 'https://support.lunet.studio/en/docs/duplicator.html'],\
				['LUnet Studio', 'https://lunet.studio']]\
		})
	elif action=='actSettings':
		winConf.show()
		app.set('cfgLang', 'text', conf['cfgLang'])
		app.set('pathData', 'text', conf['pathData'])
		app.set('pathTmpData', 'text', conf['pathTmpData'])
		app.set('pathMedia', 'text', conf['pathMedia'])
		app.set('cfgChown', 'text', conf['cfgChown'])
		app.set('cfgChmod', 'text', conf['cfgChmod'])
		app.set('cfgZipQuality', 'value', int(conf['cfgZipQuality']))
		if int(conf['cfgBeep'])==1:
			app.set('cfgBeep', 'active', True)
		else:
			app.set('cfgBeep', 'active', False)
		app.set('cfgHiddenDevs', 'text', conf['cfgHiddenDevs'])
		app.set('pathApp', 'text', conf['pathApp'])
		app.set('pathPortfs', 'text', conf['pathPortfs'])
		app.set('pathTmp', 'text', conf['pathTmp'])
		app.set('pathMnt', 'text', conf['pathMnt'])
		app.set('cmdAppLinks', 'text', conf['cmdAppLinks'])
		app.set('cfgAutomounts', 'text', c.loadFile('/etc/duplicator/automount.conf'))
	elif action=='confApply':
		newconf=app.get(['cfgLang', 'pathData', 'pathTmpData', 'pathMedia', 'cfgChown', 'cfgChmod', 'pathApp', 'pathPortfs', 'pathTmp', 'pathMnt', 'cfgHiddenDevs', 'cmdAppLinks'], 'text')
		newconf.update({'cfgZipQuality':str(int(app.get('cfgZipQuality', 'value')))})
		if app.get('cfgBeep', 'active')==True:
			newconf.update({'cfgBeep':'1'})
		else:
			newconf.update({'cfgBeep':'0'})
		cfg.save('defaults.conf', newconf)
		c.saveFile('/etc/duplicator/automount.conf', app.get('cfgAutomounts', 'text'))
		app.obj('winConf').hide()
		cw.show('message', {'severity':'info', 'title':dynglade.terms['diag2c'], 'text':dynglade.terms['diag2t']})
	elif action=='confClose':
		app.obj('winConf').hide()

def sigMenuWizards(widget, *args):
	wizard=widget.get_property('name')
	showFrame(wizard, widget.get_property('tooltip_text').replace("\n", ' '))
def sigDashWizards(widget, *args):
	wizard=widget.get_property('name')
	showFrame(wizard, app.obj('btnMenu'+str(wizard)).get_property('tooltip_text').replace("\n", ' '))

def sigActionOk(widget, *args):
	wizard=widget.get_property('name')
	if int(wizard)<40:
		cw.show('message', {'severity':'yesno', 'title':dynglade.terms['diag1c'], 'text':dynglade.terms['diag1t'], 'wizard':wizard}, retActionOk)
	else:
		retActionOk('yes', {'wizard':wizard})
def retActionOk(response, wizargs):
	if response=='yes':
		wizard=wizargs['wizard']
		if int(wizard)<40:
			thr=createThread()
		if wizard=='1':
			pargs=dict({'action':'duplicate', 'thr':thr, 'sid':threads[thr], 'source':app.obj('txt1Source').get_property('text'), 'target':app.obj('txt1Target').get_property('text'), 'beep':0, 'shdown':0, 'wipe':0, 'crop':0, 'gzip':0})
			if app.obj('chk1Props1').get_active()==True:
				pargs['beep']=1
			if app.obj('chk1Props2').get_active()==True:
				pargs['crop']=1
			if app.obj('chk1Props3').get_active()==True:
				pargs['shdown']=1
			if app.obj('chk1Props4').get_active()==True:
				pargs['gzip']=1
			if app.obj('chk1Props5').get_active()==True:
				pargs['wipe']=1
			startProcess(pargs, 'Duplicate: '+pargs['source']+' > '+pargs['target'])
		elif wizard=='2':
			pargs=dict({'action':'makeimg', 'thr':thr, 'sid':threads[thr], 'size':int(app.obj('adj2Size').get_property('value')), 'fscreate':0, 'fstype':str(app.obj('cmb2FsType').get_active_id()), 'fslabel':app.obj('txt2FsLabel').get_property('text'), 'copydata':0, 'cpsource':app.obj('dir2CopyData').get_filename(), 'target':app.obj('txt2Target').get_property('text'), 'beep':1, 'shdown':0})
			if app.obj('rad2Format1').get_active()==True:
				pargs['action']='makeimg'
			if app.obj('rad2Format2').get_active()==True:
				pargs['action']='makeiso'
			if app.obj('chk2FsCreate').get_active()==True:
				pargs['fscreate']=1
			if app.obj('chk2CopyData').get_active()==True:
				pargs['copydata']=1
			startProcess(pargs, 'Create new image file: '+pargs['target'])
		elif wizard=='40':
			app.obj('frm4Mount').set_property('visible', True)
		elif wizard=='41':
			treeSelect=app.obj('tree4Mounts').get_selection().get_selected()
			umntdev=treeSelect[0].get_value(treeSelect[1], 1)
			c.execCmd('umount '+str(umntdev))
			refreshMounts()
		elif wizard=='42':
			refreshMounts()
		elif wizard=='45':
			app.obj('frm4Mount').set_property('visible', False)
			#TODO
			refreshMounts()
			cw.show('message', {'severity':'err', 'title':'Error', 'text':'Internal error occured!'})
		else:
			if int(wizard)>29 and int(wizard)<40:
				pargs=dict({'action':'', 'thr':thr, 'sid':threads[thr], 'source':app.obj('txt3Source').get_property('text'), 'beep':0, 'shdown':0})
				if app.obj('chk3Props1').get_active()==True:
					pargs['beep']=1
				if app.obj('chk3Props2').get_active()==True:
					pargs['shdown']=1
				if wizard=='30':
					pargs['action']='crop'
					label='Crop empty space: '+pargs['source']
				elif wizard=='31':
					pargs['action']='compress'
					label='Compress image to archive: '+pargs['source']
				elif wizard=='32':
					pargs['action']='decompress'
					label='Extract image from archive: '+pargs['source']
				elif wizard=='33':
					pargs['action']='burn'
					label='Burn to disc: '+pargs['source']
				elif wizard=='34':
					pargs['action']='delete'
					label='Delete image file: '+pargs['source']
				startProcess(pargs, label)
		if int(wizard)<40:
			hideFrames()
			showFrame(0)

def sigActionCancel(widget, *args):
	wizard=widget.get_property('name')
	if int(wizard)<40:
		hideFrames()
		showFrame(0)
	else:
		if wizard=='45':
			app.obj('frm4Mount').set_property('visible', False)

def sigAbortThread(widget, *args):
	thid=int(widget.get_property('name'))
	pid=pfs.get(str(threads[thid])+'.pid')
	if int(pid)>0:
		widget.set_property('sensitive', False)
		c.execCmd(appdir+'/inc/process.sh abort '+str(threads[thid])+' '+str(pid)+' &')


def sigRunApp(widget, *args):
	c.execCmd(conf['arrAppLinks'][int(widget.get_property('name'))]+' &')


def sigWinShow(widget, *args):
	try:
		winProgress.hide()
	except:
		pass
def sigWinHide(widget, *args):
	if countThreads()>0 and int(cargs[2])<1:
		winProgress.show()


def sigChanged_txt1Source(widget, *args):
	onChange('Source', 1, 'txt')
def sigChanged_rad1Source(widget, *args):
	onChange('Source', 1, 'rad', int(widget.get_property('name')))
def sigChanged_sel1Source1(widget, *args):
	onChange('Source', 1, 'sel', 1)
def sigChanged_sel1Source2(widget, *args):
	onChange('Source', 1, 'sel', 2)
def sigChanged_sel1Source3(widget, *args):
	onChange('Source', 1, 'sel', 3)

def sigChanged_txt1Target(widget, *args):
	onChange('Target', 1, 'txt')
def sigChanged_rad1Target(widget, *args):
	onChange('Target', 1, 'rad', int(widget.get_property('name')))
def sigChanged_sel1Target1(widget, *args):
	onChange('Target', 1, 'sel', 1)
def sigChanged_sel1Target2(widget, *args):
	onChange('Target', 1, 'sel', 2)
def sigChanged_sel1Target3(widget, *args):
	onChange('Target', 1, 'sel', 3)

def sigChanged_txt2Target(widget, *args):
	onChange('Target', 2, 'txt')
def sigChanged_rad2Target(widget, *args):
	onChange('Target', 2, 'rad', int(widget.get_property('name')))
def sigChanged_sel2Target1(widget, *args):
	onChange('Target', 2, 'sel', 1)
def sigChanged_sel2Target3(widget, *args):
	onChange('Target', 2, 'sel', 3)
def sigChanged_rad2Format(widget, *args):
	ext=widget.get_property('name')
	app.obj('chk2CopyData').set_property('active', False)
	if ext=='img':
		app.obj('chk2FsCreate').set_property('sensitive', True)
		app.obj('chk2FsCreate').set_property('active', False)
		app.obj('chk2CopyData').set_property('sensitive', False)
	elif ext=='iso':
		app.obj('chk2FsCreate').set_property('sensitive', False)
		app.obj('chk2FsCreate').set_property('active', True)
		app.obj('chk2CopyData').set_property('sensitive', True)
	app.obj('cmb2FsType').set_property('sensitive', False)
	app.obj('txt2FsLabel').set_property('sensitive', False)
	app.obj('dir2CopyData').set_property('sensitive', False)
def sigChanged_chk2FsCreate(widget, *args):
	isact=widget.get_property('active')
	if isact==True:
		app.obj('cmb2FsType').set_property('sensitive', True)
		app.obj('txt2FsLabel').set_property('sensitive', True)
		app.obj('chk2CopyData').set_property('sensitive', True)
	else:
		app.obj('cmb2FsType').set_property('sensitive', False)
		app.obj('txt2FsLabel').set_property('sensitive', False)
		app.obj('chk2CopyData').set_property('sensitive', False)
def sigChanged_chk2CopyData(widget, *args):
	isact=widget.get_property('active')
	if isact==True:
		app.obj('dir2CopyData').set_property('sensitive', True)
	else:
		app.obj('dir2CopyData').set_property('sensitive', False)

def sigChanged_txt3Source(widget, *args):
	onChange('Source', 3, 'txt')
def sigChanged_rad3Source(widget, *args):
	onChange('Source', 3, 'rad', int(widget.get_property('name')))
def sigChanged_sel3Source1(widget, *args):
	onChange('Source', 3, 'sel', 1)
def sigChanged_sel3Source2(widget, *args):
	onChange('Source', 3, 'sel', 2)
def sigChanged_sel3Source3(widget, *args):
	onChange('Source', 3, 'sel', 3)

def sigChanged_txt4Source(widget, *args):
	onChange('Source', 4, 'txt')
def sigChanged_rad4Source(widget, *args):
	onChange('Source', 4, 'rad', int(widget.get_property('name')))
def sigChanged_sel4Source1(widget, *args):
	onChange('Source', 4, 'sel', 1)
def sigChanged_sel4Source2(widget, *args):
	onChange('Source', 4, 'sel', 2)
def sigChanged_sel4Source3(widget, *args):
	onChange('Source', 4, 'sel', 3)
def sigChanged_rad4s4Type(widget, *args):
	onChange('Source', 4, 'rad', 4)



##################################################
# Funkcie

def onInit():
	app.set('winMain', 'title', APPTITLE)
	hideFrames()
	showFrame(0)
	i=1
	while i<len(threads):
		destroyThread(i)
		i=i+1
	app.obj('boxProgress').hide()


def getSID():
	cmd=c.execCmd('date +"%N"', True)
	sID=cmd['stdout']
	return sID


def hideFrames():
	i=0
	while i<5:
		app.obj('grdWizard_'+str(i)).hide()
		i=i+1

def showFrame(frame, title=''):
	hideFrames()
	app.obj('grdWizard_'+str(frame)).show()
	if title!='':
		app.obj('lblWizard').set_property('label', title)
	if frame>0:
		refreshCombos()
		refreshMounts()


def changePrgLabel(label_object, text=''):
	txt=app.obj(label_object).get_property('label').split("\n")
	return str(txt[0]+"\n"+text)


def refreshCombos():
	app.obj('sel1Source1').remove_all()
	app.obj('sel1Source2').remove_all()
	app.obj('sel1Target2').remove_all()
	app.obj('sel3Source1').remove_all()
	app.obj('sel3Source2').remove_all()
	app.obj('sel4Source1').remove_all()
	app.obj('sel4Source2').remove_all()
	app.obj('cmb2FsType').remove_all()
	f=[]
	for (dirpath, dirnames, filenames) in os.walk(conf['pathData']):
		f.extend(filenames)
		break
	f.sort()
	i=0
	while i<len(f):
		app.obj('sel1Source1').insert(-1, f[i], f[i])
		app.obj('sel3Source1').insert(-1, f[i], f[i])
		app.obj('sel4Source1').insert(-1, f[i], f[i])
		i=i+1
	
	#lsblk -J -o NAME,TYPE,MODEL,LABEL,UUID,FSTYPE,MOUNTPOINT,SIZE,STATE
	#TODO
	blkset=c.execCmd('lsblk -nPd -o NAME,MODEL,LABEL,TYPE,SIZE')
	blkset['stdout']=blkset['stdout'].replace('NAME="', '')
	blkset['stdout']=blkset['stdout'].replace('" MODEL="', '|')
	blkset['stdout']=blkset['stdout'].replace('" LABEL="', '|')
	blkset['stdout']=blkset['stdout'].replace('" TYPE="', '|')
	blkset['stdout']=blkset['stdout'].replace('" SIZE="', '|')
	blkset['stdout']=blkset['stdout'].replace('"', '')
	blks=blkset['stdout'].split("\n")
	i=0
	while i<len(blks)-1:
		blk=blks[i].split('|')
		rwshow=0
		if blk[0] not in conf['arrHiddenDevs']:
			rwshow=1
		blkt=blk[0]+'  ---  '+blk[3]+': '+blk[1]+'  ---  '+blk[4]
		app.obj('sel1Source2').insert(-1, blk[0], blkt)
		if rwshow>0:
			app.obj('sel3Source2').insert(-1, blk[0], blkt)
			app.obj('sel1Target2').insert(-1, blk[0], blkt)
		blkset=c.execCmd('lsblk -nP -o NAME,LABEL,MOUNTPOINT,TYPE,SIZE /dev/'+blk[0])
		blkset['stdout']=blkset['stdout'].replace('NAME="', '')
		blkset['stdout']=blkset['stdout'].replace('" LABEL="', '|')
		blkset['stdout']=blkset['stdout'].replace('" MOUNTPOINT="', '|')
		blkset['stdout']=blkset['stdout'].replace('" TYPE="', '|')
		blkset['stdout']=blkset['stdout'].replace('" SIZE="', '|')
		blkset['stdout']=blkset['stdout'].replace('"', '')
		blks2=blkset['stdout'].split("\n")
		y=1
		while y<len(blks2)-1:
			blk=blks2[y].split('|')
			blkt='|--'+blk[0]+'  ---  '+blk[3]+': '+blk[1]+'  >  '+blk[2]+'  ---  '+blk[4]
			app.obj('sel1Source2').insert(-1, blk[0], blkt)
			app.obj('sel4Source2').insert(-1, blk[0], blkt)
			if rwshow>0:
				app.obj('sel3Source2').insert(-1, blk[0], blkt)
				app.obj('sel1Target2').insert(-1, blk[0], blkt)
			y=y+1
		i=i+1
	
	fsset=c.execCmd('ls -1 /sbin/mkfs.*')
	fsset['stdout']=fsset['stdout'].replace('/sbin/mkfs.', '')
	fss=fsset['stdout'].split("\n")
	i=0
	while i<len(fss)-1:
		app.obj('cmb2FsType').insert(-1, fss[i], fss[i])
		i=i+1
def refreshMounts():
	app.obj('lst4Mounts').clear()
	#findmnt -J -o SOURCE,TARGET,FSTYPE,OPTIONS,LABEL,UUID,SIZE,USE%,AVAIL,USED
	#TODO
	mntset=c.execCmd('mount')
	mnts=mntset['stdout'].split("\n")
	i=0
	while i<len(mnts)-1:
		mnts[i]=mnts[i].replace(' on ', '|')
		mnts[i]=mnts[i].replace(' type ', '|')
		mnts[i]=mnts[i].replace(' (', '|')
		mnt=mnts[i].split('|')
		mnt[3]=mnt[3].replace(',', ', ')
		mnt[3]=mnt[3].replace(')', '')
		app.obj('lst4Mounts').append([mnt[0], mnt[1], mnt[2], 'label', 'size', mnt[3]])
		i=i+1


def createThread():
	free=0
	i=0
	while i<len(threads):
		if threads[i]==0 and free==0:
			free=i
		i=i+1
	if free>0:
		app.obj('boxProgress').show()
		app.obj('boxProgress'+str(free)).show()
		app.obj('boxProgress'+str(free+10)).show()
		threads[free]=getSID()
	return free

def changeThread(thread):
	return thread

def destroyThread(thread):
	app.obj('boxProgress'+str(thread)).hide()
	app.obj('boxProgress'+str(thread+10)).hide()
	threads[thread]=0
	return True

def countThreads():
	thlen=0
	i=0
	while i<len(threads):
		if threads[i]>0:
			thlen=thlen+1
		i=i+1
	return thlen-1


def onChange(box, bid, item, iid=0):
	if item=='txt':
		app.obj('rad'+str(bid)+box+'3').set_property('active', True)
	elif item=='rad' or item=='sel':
		outText=''
		if item=='sel':
			app.obj('rad'+str(bid)+box+str(iid)).set_property('active', True)
		if iid==1:
			if box=='Source':
				outText=conf['pathData']+'/'+str(app.obj('sel'+str(bid)+box+str(iid)).get_active_id())
			elif box=='Target':
				outText=conf['pathData']+'/'+str(app.obj('sel'+str(bid)+box+str(iid)).get_property('text')+'.img')
				if str(app.obj('sel'+str(bid)+box+str(iid)).get_property('text'))=='':
					outText=conf['pathData']+'/NewDevice_'+time.strftime("%Y%m%d-%H%M%S")+'.img'
				app.obj('chk1Props2').set_property('sensitive', True)
				app.obj('chk1Props2').set_property('active', True)
				app.obj('chk1Props4').set_property('sensitive', True)
		elif iid==2:
			outText='/dev/'+str(app.obj('sel'+str(bid)+box+str(iid)).get_active_id())
			if box=='Target':
				app.obj('chk1Props2').set_property('sensitive', False)
				app.obj('chk1Props2').set_property('active', False)
				app.obj('chk1Props4').set_property('sensitive', False)
				app.obj('chk1Props4').set_property('active', False)
		elif iid==3:
			if box=='Source':
				outText=str(app.obj('sel'+str(bid)+box+str(iid)).get_filename())
			elif box=='Target':
				outText=str(app.obj('sel'+str(bid)+box+str(iid)).get_filename()+'/NewDevice_'+time.strftime("%Y%m%d-%H%M%S")+'.img')
				app.obj('chk1Props2').set_property('sensitive', True)
				app.obj('chk1Props2').set_property('active', True)
				app.obj('chk1Props4').set_property('sensitive', True)
		app.obj('txt'+str(bid)+box).set_property('text', outText)


def startProcess(args, label):
	app.obj('prgBar'+str(args['thr'])).show()
	app.obj('prgImg'+str(args['thr'])).hide()
	app.obj('prgBtn'+str(args['thr'])).set_property('sensitive', True)
	app.obj('prgLbl'+str(args['thr'])).set_property('label', '<small><i>'+time.strftime('%d/%m/%y %H:%M:%S')+'</i> | '+label+"</small>\n"+'<b>0/0 : N/A</b>')
	app.obj('prgBar'+str(args['thr']+10)).show()
	app.obj('prgImg'+str(args['thr']+10)).hide()
	app.obj('prgLbl'+str(args['thr']+10)).set_property('label', app.obj('prgLbl'+str(args['thr'])).get_property('label'))
	acts=[{'c':'init', 't':'Initializing...', 'a':''}]
	abort=False
	
	if args['action']=='duplicate':
		if args['source']=='' or args['target']=='':
			abort=True
		else:
			if args['wipe']==1 and args['target'][0:4]=='/dev':
				acts.extend([{'c':'ddio', 't':'Wiping device...', 'a':'/dev/zero "'+args['target']+'"'}])
			if args['source'][len(args['source'])-3:len(args['source'])]=='.gz':
				acts.extend([{'c':'gunzip', 't':'Decompressing archive...', 'a':'"'+args['source']+'"'}])
				src=args['source'].split('/')
				args['source']=conf['pathTmpData']+'/'+str(args['sid'])+'_'+src[len(src)-1].replace('.gz', '')
			acts.extend([{'c':'ddio', 't':'Copying data...', 'a':'"'+args['source']+'" "'+args['target']+'"'}])
			if args['crop']==1 and args['target'][0:4]!='/dev':
				acts.extend([{'c':'crop', 't':'Trimming empty space...', 'a':'"'+args['target']+'"'}])
			if args['gzip']==1 and args['target'][0:4]!='/dev':
				acts.extend([{'c':'gzip', 't':'Compressing image file...', 'a':'"'+args['target']+'"'}])
			acts.extend([{'c':'clean', 't':'Cleaning temporary files...', 'a':''}])
	elif args['action']=='makeimg':
		if args['target']=='':
			abort=True
		else:
			acts.extend([{'c':'ddio', 't':'Sizing image...', 'a':'/dev/zero "'+args['target']+'" bs=1024,count=0,seek='+str(args['size']*1024)}])
			if args['fscreate']==1 and args['fstype']!='':
				if args['fslabel']=='':
					args['fslabel']='IMG'+time.strftime("%Y%m%d")
				acts.extend([{'c':'mkfs', 't':'Creating filesystem...', 'a':'"'+args['target']+'" '+args['fstype']+' "'+args['fslabel']+'"'}])
			if args['copydata']==1 and args['cpsource']!='':
				acts.extend([{'c':'dtofs', 't':'Copying data...', 'a':'"'+args['cpsource']+'" "'+args['target']+'"'}])
	elif args['action']=='crop':
		if args['source']=='':
			abort=True
		else:
			acts.extend([{'c':'crop', 't':'Trimming empty space...', 'a':'"'+args['source']+'"'}])
	elif args['action']=='compress':
		if args['source']=='':
			abort=True
		else:
			acts.extend([{'c':'gzip', 't':'Compressing image file...', 'a':'"'+args['source']+'"'}])
			acts.extend([{'c':'clean', 't':'Cleaning temporary files...', 'a':''}])
	elif args['action']=='decompress':
		if args['source']=='':
			abort=True
		else:
			acts.extend([{'c':'gunzipimg', 't':'Decompressing archive...', 'a':'"'+args['source']+'"'}])
			acts.extend([{'c':'clean', 't':'Cleaning temporary files...', 'a':''}])
	elif args['action']=='burn':
		if args['source']=='':
			abort=True
		else:
			#TODO
			acts.extend([{'c':'clean', 't':'Cleaning temporary files...', 'a':''}])
	elif args['action']=='delete':
		if args['source']=='':
			abort=True
		else:
			acts.extend([{'c':'delimg', 't':'Removing file...', 'a':'"'+args['source']+'"'}])
	
	if abort==True:
		app.obj('prgBar'+str(args['thr'])).hide()
		app.obj('prgImg'+str(args['thr'])).set_property('stock', 'gtk-quit')
		app.obj('prgImg'+str(args['thr'])).show()
		app.obj('prgBtn'+str(args['thr'])).set_property('sensitive', False)
		app.obj('prgLbl'+str(args['thr'])).set_property('label', changePrgLabel('prgLbl'+str(args['thr']), '<b>Operation failed!</b>'))
		app.obj('prgBar'+str(args['thr']+10)).hide()
		app.obj('prgImg'+str(args['thr']+10)).set_property('stock', 'gtk-quit')
		app.obj('prgImg'+str(args['thr']+10)).show()
		app.obj('prgLbl'+str(args['thr']+10)).set_property('label', app.obj('prgLbl'+str(args['thr'])).get_property('label'))
	else:
		if args['shdown']==1:
			acts.extend([{'c':'shdown', 't':'Computer is going to shutdown...', 'a':''}])
		actions.update({str(args['sid']):[-1, 1, 1, acts, args['beep']]})


def ctrlProcesses():
	i=1
	while i<len(threads):
		if threads[i]>1:
			if actions.has_key(str(threads[i])):
				sid=str(threads[i])
				if actions[sid][1]==actions[sid][2]:
					actions[sid][0]=actions[sid][0]+1
					sp=actions[sid][0]
					sa=actions[sid][3]
					if sp>len(sa)-1:
						if sp<999:
							app.obj('prgBar'+str(i)).hide()
							app.obj('prgImg'+str(i)).set_property('stock', 'gtk-ok')
							app.obj('prgImg'+str(i)).show()
							app.obj('prgBtn'+str(i)).set_property('sensitive', False)
							app.obj('prgLbl'+str(i)).set_property('label', changePrgLabel('prgLbl'+str(i), '<b>Done.</b>'))
							app.obj('prgBar'+str(i+10)).hide()
							app.obj('prgImg'+str(i+10)).set_property('stock', 'gtk-ok')
							app.obj('prgImg'+str(i+10)).show()
							app.obj('prgLbl'+str(i+10)).set_property('label', app.obj('prgLbl'+str(i)).get_property('label'))
						else:
							app.obj('prgBar'+str(i)).hide()
							app.obj('prgImg'+str(i)).set_property('stock', 'gtk-quit')
							app.obj('prgImg'+str(i)).show()
							app.obj('prgBtn'+str(i)).set_property('sensitive', False)
							app.obj('prgLbl'+str(i)).set_property('label', changePrgLabel('prgLbl'+str(i), '<b>Operation failed!</b>'))
							app.obj('prgBar'+str(i+10)).hide()
							app.obj('prgImg'+str(i+10)).set_property('stock', 'gtk-quit')
							app.obj('prgImg'+str(i+10)).show()
							app.obj('prgLbl'+str(i+10)).set_property('label', app.obj('prgLbl'+str(i)).get_property('label'))
						if actions[sid][2]>-1:
							actions[sid][1]=0
							actions[sid][2]=-1
							if actions[sid][4]==1:
								c.execCmd(appdir+'/inc/process.sh pcbeep &')
					else:
						c.execCmd(appdir+'/inc/process.sh '+sa[sp]['c']+' '+sid+' '+sa[sp]['a']+' &')
						time.sleep(0.5)
						actions[sid][1]=0
						actions[sid][2]=1
						app.obj('prgLbl'+str(i)).set_property('label', changePrgLabel('prgLbl'+str(i), '<b>'+str(sp+1)+'/'+str(len(sa))+' : '+sa[sp]['t']+'</b>'))
						app.obj('prgLbl'+str(i+10)).set_property('label', app.obj('prgLbl'+str(i)).get_property('label'))
				else:
					if actions[sid][2]==-1:
						actions[sid][1]=actions[sid][1]+1
						if actions[sid][1]==180:
							destroyThread(i)
					else:
						sp=actions[sid][0]
						sa=actions[sid][3]
						status=pfs.get(str(sid)+'.out').split('|')
						actions[sid][1]=0
						actions[sid][2]=1
						stattxt=''
						try:
							if status[0]!='' and status[0]!='x':
								actions[sid][1]=int(status[0])
							if status[1]!='' and status[1]!='x':
								actions[sid][2]=int(status[1])
							if status[0]=='x' or status[1]=='x':
								actions[sid][0]=999
								actions[sid][1]=0
								actions[sid][2]=0
						except:
							pass
						if len(status)>2 and str(status[2])!='':
							stattxt=' <small>('+str(status[2])+')</small>'
						app.obj('prgLbl'+str(i)).set_property('label', changePrgLabel('prgLbl'+str(i), '<b>'+str(sp+1)+'/'+str(len(sa))+' : '+sa[sp]['t']+'</b>'+stattxt))
						app.obj('prgLbl'+str(i+10)).set_property('label', app.obj('prgLbl'+str(i)).get_property('label'))
		i=i+1
	return True




##################################################
## Hlavna procedura                             ##
##################################################

winMain=app.newWindow('winMain')
cw=CommonWindows(app)

app.handleSignals(['runApp'], sigRunApp)

app.handleSignals(['sigWinShow'], sigWinShow)
app.handleSignals(['sigWinHide'], sigWinHide)

app.handleSignals(['btnMenu_clicked'], sigMenuActions)
app.handleSignals(['btnMenuWizard_clicked'], sigMenuWizards)
app.handleSignals(['dashMenuWizard_clicked'], sigDashWizards)
app.handleSignals(['btnOk_clicked'], sigActionOk)
app.handleSignals(['btnCancel_clicked'], sigActionCancel)

app.handleSignals(['prgCancel_clicked'], sigAbortThread)

app.handleSignals(['changed_txt1Source'], sigChanged_txt1Source)
app.handleSignals(['changed_rad1Source'], sigChanged_rad1Source)
app.handleSignals(['changed_sel1Source1'], sigChanged_sel1Source1)
app.handleSignals(['changed_sel1Source2'], sigChanged_sel1Source2)
app.handleSignals(['changed_sel1Source3'], sigChanged_sel1Source3)

app.handleSignals(['changed_txt1Target'], sigChanged_txt1Target)
app.handleSignals(['changed_rad1Target'], sigChanged_rad1Target)
app.handleSignals(['changed_sel1Target1'], sigChanged_sel1Target1)
app.handleSignals(['changed_sel1Target2'], sigChanged_sel1Target2)
app.handleSignals(['changed_sel1Target3'], sigChanged_sel1Target3)

app.handleSignals(['changed_txt2Target'], sigChanged_txt2Target)
app.handleSignals(['changed_rad2Target'], sigChanged_rad2Target)
app.handleSignals(['changed_sel2Target1'], sigChanged_sel2Target1)
app.handleSignals(['changed_sel2Target3'], sigChanged_sel2Target3)
app.handleSignals(['changed_rad2Format'], sigChanged_rad2Format)
app.handleSignals(['changed_chk2FsCreate'], sigChanged_chk2FsCreate)
app.handleSignals(['changed_chk2CopyData'], sigChanged_chk2CopyData)

app.handleSignals(['changed_txt3Source'], sigChanged_txt3Source)
app.handleSignals(['changed_rad3Source'], sigChanged_rad3Source)
app.handleSignals(['changed_sel3Source1'], sigChanged_sel3Source1)
app.handleSignals(['changed_sel3Source2'], sigChanged_sel3Source2)
app.handleSignals(['changed_sel3Source3'], sigChanged_sel3Source3)

app.handleSignals(['changed_txt4Source'], sigChanged_txt4Source)
app.handleSignals(['changed_rad4Source'], sigChanged_rad4Source)
app.handleSignals(['changed_sel4Source1'], sigChanged_sel4Source1)
app.handleSignals(['changed_sel4Source2'], sigChanged_sel4Source2)
app.handleSignals(['changed_sel4Source3'], sigChanged_sel4Source3)
app.handleSignals(['changed_rad4s4Type'], sigChanged_rad4s4Type)

app.connectSignals()


app.newTimer(500, ctrlProcesses)

onInit()


winConf=app.newWindow('winConf', False, False)
winProgress=app.newWindow('winProgress', False, False)


app.start()

dynglade.flush()
pfs.destroy()
