#!/bin/bash

source /etc/duplicator/defaults.conf

sID="$2"

runFile="$pathPortfs/$sID.run.val"
outFile="$pathPortfs/$sID.out.val"
pidFile="$pathPortfs/$sID.pid.val"

valPs=`ps -Af`
linePid=`echo "$valPs" | grep "process.sh" | grep "$sID"`
if [ "$linePid" != "" ]; then
	arrPid=($linePid)
	valPid="${arrPid[1]}"
	echo "$valPid" > $pidFile
fi

case "$1" in
  pcbeep)
	if [ "$3" == "short" ]; then
		if [ "$cfgBeep" == "1" ]; then
			beep -f 500 -n -f 700 -n -f 900 -n -f 700 -l 300 &
		else
			aplay ${pathApp}/share/done_S.wav &
		fi
	else
		if [ "$cfgBeep" == "1" ]; then
			beep -f 500 -n -f 700 -n -f 900 -n -f 600 -n -f 800 -n -f 1000 -n -f 700 -l 300 &
		else
			aplay ${pathApp}/share/done_L.wav &
		fi
	fi
	;;

  init)
	echo "0|1|" > $outFile
	sleep 1
	echo "1|1|" > $outFile
	;;

  ddio)
	valIf="$3"
	valOf="$4"
	valArgs=""
	if [ "$5" != "" ]; then
		valArgs="$5"
		valArgs=" ${valArgs//,/ }"
	fi
	echo "0|2|" > $outFile
	dd if=$valIf of=${valOf}${valArgs} > $runFile 2>&1 &
	sleep 2
	valOut="0";
	valPs=`ps -Af`
	linePid=`echo "$valPs" | grep "dd if=$valIf of=$valOf"`
	if [ "$linePid" != "" ]; then
		arrPid=($linePid)
		valPid="${arrPid[1]}"
		echo "$valPid" > $pidFile
	fi
	while [ "$valOut" != "" ]; do
		valOut=`echo "$valPs" | grep "dd if=$valIf of=$valOf"`
		kill -USR1 $valPid > /dev/null 2>&1
		valTail=`tail --lines=1 -q $runFile`
		if [ "$valTail" != "" ]; then
			echo "1|2|${valTail:0:62}" > $outFile
			echo "" > $runFile
		fi
		valPs=`ps -Af`
		sleep 1
	done
	chown $cfgChown $valOf
	chmod $cfgChmod $valOf
	echo "2|2|" > $outFile
	;;

  crop)
	valOf="$3"
	echo "0|2|" > $outFile
	valFdisk=`fdisk -l $valOf | tail --lines=1`
	valFdisk="${valFdisk//"${valOf}"/PART:}"
	valFdisk="${valFdisk//\*/}"
	arrFdisk=($valFdisk)
	if [ "${arrFdisk[2]}" == "" ]; then
		echo "x|2|" > $outFile
	else
		truncate --size=$[(${arrFdisk[2]}+4)*512] $valOf 2>&1 &
		valOut="0";
		valPs=`ps -Af`
		while [ "$valOut" != "" ]; do
			valOut=`echo "$valPs" | grep "truncate" | grep "$valOf"`
			echo "1|2|" > $outFile
			valPs=`ps -Af`
			sleep 1
		done
		echo "2|2|" > $outFile
	fi
	;;

  gzip)
	valOf="$3"
	echo "0|4|" > $outFile
	valIn="$valOf"
	valIn="${valIn// /_}"
	valIn="${valIn//\// }"
	arrIn=($valIn)
	len=${#arrIn[*]}
	i=`expr $len - 1`
	
	echo "1|4|" > $outFile
	valFile="${arrIn[$i]}"
	valFile="$pathTmpData/${sID}_$valFile"
	cp -f $valOf $valFile
	sleep 1
	
	echo "2|4|" > $outFile
	gzip -qc${cfgZipQuality} $valFile > $valOf.gz
	sleep 1
	chown $cfgChown $valOf.gz
	chmod $cfgChmod $valOf.gz
	
	echo "3|4|" > $outFile
	rm -f $valOf
	sleep 1
	
	echo "4|4|" > $outFile
	;;

  gunzip | gunzipimg)
	if [ "$1" == "gunzip" ]; then
		cntTotal="2"
	else
		cntTotal="4"
	fi
	valIf="$3"
	echo "0|${cntTotal}|" > $outFile
	valIn="$valIf"
	valIn="${valIn// /_}"
	valIn="${valIn//\// }"
	arrIn=($valIn)
	len=${#arrIn[*]}
	i=`expr $len - 1`
	
	echo "1|${cntTotal}|" > $outFile
	valFile="${arrIn[$i]}"
	valFile="$pathTmpData/${sID}_${valFile//.gz/}"
	gunzip -fc $valIf > $valFile
	sleep 1
	
	if [ "$1" == "gunzipimg" ]; then
		echo "2|${cntTotal}|" > $outFile
		mv -f $valFile "${valIf//.gz/}"
		sleep 1
		
		echo "3|${cntTotal}|" > $outFile
		rm -f $valIf
		sleep 1
	fi
	
	valIf="$valFile"
	echo "${cntTotal}|${cntTotal}|" > $outFile
	;;

  mkfs)
	valOf="$3"
	valFs="$4"
	valLbl="$5"
	echo "0|2|" > $outFile
	sleep 1
	echo "1|2|" > $outFile
	mkfs.${valFs} -F -L "$valLbl" $valOf > $runFile
	echo "1|2|" > $outFile
	sleep 1
	echo "2|2|" > $outFile
	;;

  dtofs)
	valIf="$3"
	valOf="$4"
	echo "0|4|" > $outFile
	sleep 1
	mkdir $pathTmpData/${sID}.mnt 2>&1
	mount $valOf $pathTmpData/${sID}.mnt 2>&1
	sleep 1
	valMnt=`mount | grep "$valOf"`
	if [ "$valMnt" == "" ]; then
		echo "x|4|" > $outFile
	else
		echo "1|4|" > $outFile
		cp -pRf ${valIf}/* $pathTmpData/${sID}.mnt > $runFile
		echo "2|4|" > $outFile
		sleep 1
		umount $pathTmpData/${sID}.mnt 2>&1
		echo "3|4|" > $outFile
		sleep 1
		echo "4|4|" > $outFile
	fi
	;;

  delimg)
	valIf="$3"
	echo "0|2|" > $outFile
	sleep 1
	echo "1|2|" > $outFile
	rm -f $valIf 2>&1
	echo "1|2|" > $outFile
	sleep 1
	echo "2|2|" > $outFile
	;;

  clean)
	echo "0|2|" > $outFile
	sleep 1
	echo "1|2|" > $outFile
	rm -f $pathTmpData/${sID}_* 2>&1
	echo "1|2|" > $outFile
	sleep 1
	echo "2|2|" > $outFile
	;;

  shdown)
	echo "0|1|" > $outFile
	sleep 10
	echo "1|1|" > $outFile
	sleep 20
	shutdown -h now &
	;;

  abort)
	kill $3
	kill -s SIGKILL $3
	echo "x|1|" > $outFile
	valPs=`ps -Af`
	linePid=`echo "$valPs" | grep "$3" | grep -v "process.sh"`
	while [ "$linePid" != "" ]; do
		arrPid=($linePid)
		valPid="${arrPid[1]}"
		kill $valPid
		kill -s SIGKILL $valPid
		valPs=`ps -Af`
		linePid=`echo "$valPs" | grep "$3" | grep -v "process.sh"`
	done
	;;

  *)
	echo "x|0|" > $outFile
	;;
esac
